/*
 * Copyright (c) 2015-2017 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Tue Jun  2 2015
 */
#ifndef H_LIBGENOMIX
#define H_LIBGENOMIX

#include <stddef.h>
#include <stdint.h>

#if defined(_WIN32) || defined(_WIN64)
# include <winsock2.h>
typedef SOCKET	socket_s;
#else
typedef int	socket_s;
# define INVALID_SOCKET	-1
# define SOCKET_ERROR	-1
#endif

#include "mex.h"

enum http_state {
  http_status, http_header, http_body, http_eof, http_ok
};

struct http_buffer {
  char *data, *start, *end;
  size_t len;
};

struct http_rqst {
  enum http_state state;
  uint16_t status;
  ssize_t content_length;
  char *body;

  struct http_buffer uri, buffer, args;
};

struct http_event {
  size_t event;
  size_t data;

  struct http_buffer buffer;
};

void		genomix_event(const char *data);

int		genomix_connect(char *host, char *session, char *emsg,
                        size_t emsgsize);
int		genomix_disconnect(int id);
int		genomix_get(int id, const char *cmd, const char *args[],
                        char *emsg, size_t emsgsize);
const char *	genomix_reply(int id, char *emsg, size_t emsgsize);
void		genomix_doevent(int timeout);

int		http_get(socket_s fd, struct http_rqst *rqst,
                        const char *session, const char *cmd,
                        const char *args[]);
int		http_reply(socket_s fd, struct http_rqst *rqst);
int		http_event(socket_s fd, struct http_event *evt);
int		http_transaction(socket_s fd, struct http_rqst *rqst,
                        const char *method, const char *uri, const char *body);

mxArray *	j2s_value(const char **json, char *emsg,
                          size_t emsgsize);
char *		s2j_input(const mxArray *input, char *emsg,
                          size_t emsgsize);
int		s2j_value(const mxArray *value, char **json, char **jend,
                        size_t *jlen);

#endif /* H_LIBGENOMIX */
